#!/bin/bash
#
# Script to configure a new ISP connection and other stuff on RouterTech firmware
# This is useful when a new firmware release has been flashed, and you want to 
# reconfigure it without going through the web interface
#
# **** This script requires RouterTech firmware v2.95 or later ****
#
# This script will also run some other scripts, if it finds them:
#	/var/user.sh
#	/nvram/user.sh
#
# So you can store the commands to restore all your favourite settings in the 
# minix partition ("/nvram/user.sh") and this script run that script to restore them
#
# **** NOTE ****: ensure that all scripts and text files are in unix format
#
# ISP and other configuration settings can be supplied at the command line, or stored in
#	/var/user.conf
#	/nvram/user.conf
#
# If stored in the correct format (see the help screen for a sample user.conf file) 
# they will be loaded and used by the script, meaning that no parameters need be 
# supplied at the command line
#
# All the usual disclaimers apply. Use at your own peril.
#
# (c)2010-2011, RouterTech
# Author: TheChief
# Last amended: 8 January 2011
#

. /usr/local/bin/rt_utils.sh

# help screen
USAGE()
{
cat <<EOF
Usage:
$ARGV0 --isp_username=<ISP username> --isp_password=<ISP password> [optional arguments]
 
 Available optional arguments for $ARGV0:
 
 GENERAL:
   --sleep=<sleep interval (seconds)>	[default=5]
   --verbose=<value>			[1=enable; default=0] 
   --enable_msntp=<value>		[1=enable; default=0]
   --enable_ipaccount=<value>		[1=enable; default=1]
   --enable_wlan=<value>		[1=enable; default=1]
   --enable_ddns=<value>		[1=enable; default=1]

 WIRELESS:   
   --wpacode=<WPA code>			[passphrase for WPA]		
   --ssid=<ssid>			[SSID for wireless AP (encoded)]		
   NOTE: the SSID is encoded. To see the current value and how it is encoded, run
         echo "ap:settings/ssid" | cm_cli_ex   # standard wireless
         or
         echo "ap:settings/ssid0/ssid" | cm_cli_ex  # 1350A wireless

 DDNS:
   --ddns_domain=<DDNS domain name>
   --ddns_username=<DDNS username>
   --ddns_password=<DDNS password>


 All these values can be entered in variables in a text file ("/var/user.conf" 
 and/or "/nvram/user.conf") instead of at the command line
       
 # Sample user.conf:
 VERBOSE="1"   # enable
 SLEEP="10"    # 10-second sleep intervals between commands
 msntp="0"     # disable
 ipaccount="1" # enable
 
 # isp: this is compulsory
 ISP_USERNAME="foo@myisp.domain.com"
 ISP_PASSWORD="foo_bar"
 
 # wireless
 wlan"=1"
 SSID="235356673243212323"
 WPACODE="fsd2R2Qsfsds@£43"
 
 # ddns
 ddns="1"
 DDNS_DOMAIN="myfoobar.noip.com"
 DDNS_USERNAME="foobarfoo"
 DDNS_PASSWORD="FooBar3642Foo21QaLK"

EOF
}

# help
if [ "$(is_parameter --help)" = "1" ]; then
  USAGE
  exit 1
fi

#####################################
#### user-configurable settings #####
#####################################

# whether to report when there is a valid connection (set to 1 to enable)
VERBOSE=$(get_opt_strict "--verbose")

# isp info
ISP_USERNAME=$(get_opt_strict "--isp_username")
ISP_PASSWORD=$(get_opt_strict "--isp_password")

# wpa info
SSID=$(get_opt_strict "--ssid")
WPACODE=$(get_opt_strict "--wpacode")

# ddns info
DDNS_DOMAIN=$(get_opt_strict "--ddns_domain")
DDNS_USERNAME=$(get_opt_strict "--ddns_username")
DDNS_PASSWORD=$(get_opt_strict "--ddns_password")

# enable or disable these?
ipaccount=$(get_opt_strict "--enable_ipaccount")
msntp=$(get_opt_strict "--enable_msntp")
ddns=$(get_opt_strict "--enable_ddns")
wlan=$(get_opt_strict "--enable_wlan")

#  sleep interval - needs to be high enough
SLEEP=$(get_opt_strict "--sleep")
[ -z "$SLEEP" ] && SLEEP=5

#########################################
### end of user-configurable settings ###
#########################################
# the core logic command to use 
#CMD=cm_command
CMD=cm_command_ex_check

#########################################
############     functions   ############
#########################################

# pppoa key name
PPPOA_KEY="encaps0:pppoa:settings"

# isp connection
do_ISP()
{
  date
  [ -z "$ISP_USERNAME" ] || [ -z "$ISP_PASSWORD" ] && {
    LOG_MSG "Insufficient information to configure the ISP connection. Quitting."
    exit 1
  }
  $CMD "$PPPOA_KEY/username=$ISP_USERNAME"
  sleep $SLEEP
  
  $CMD "$PPPOA_KEY/password=$ISP_PASSWORD"
  sleep $SLEEP
  
  # wait for WAN connection to be established
  waitforwan
  sleep $SLEEP
}


# enable and configure DDNS
do_DDNS()
{
  date
  [ "$1" = "0" ] && return # it is disabled by default

  [ -z "$DDNS_USERNAME" ] || [ -z "$DDNS_PASSWORD" ] || [ -z "$DDNS_DOMAIN" ] && {
    LOG_MSG "Insufficient information to configure DDNS."
    return
  }
  [ $ROHS -eq 1 ] && tmp=ddnsclient || tmp=ddns
  DDNS_KEY="$tmp:settings/connection0/dyndns"

  $CMD "$DDNS_KEY/state=on"
  sleep $SLEEP

  $CMD "$DDNS_KEY/domainname=$DDNS_DOMAIN"
  sleep $SLEEP

  $CMD "$DDNS_KEY/password=$DDNS_PASSWORD"
  sleep $SLEEP

  $CMD "$DDNS_KEY/username=$DDNS_USERNAME"
  sleep $SLEEP

  ppp_restore.sh
  sleep $SLEEP
}


# enable ipaccount
do_IPACCOUNT()
{
  date
  [ "$1" = "0" ] && return # it is disabled by default
  $CMD "account:settings/state=1"
  sleep $SLEEP
}


# disable msntp
do_MSNTP()
{
  date
  [ "$1" = "1" ] && return # it is enabled by default
  setenv time_servers "ntp2b.mcc.ac.uk ntp2d.mcc.ac.uk time-a.nist.gov nist1-ny.ustiming.org"
  setenv TZ GMT
  setenv setdate_enable 1
  sleep 1
  $CMD "sntp:settings/status=0"
  sleep $SLEEP
  setdate
  sleep $SLEEP
}

# enable and configure wireless
do_WLAN()
{  
  date
  # return for non-wireless, or if we receive zero (meaning don't enable)
  [ "$BOARD" = "AR7RD" ] || [ "$1" = "0" ] && return  

  [ -z "$SSID" ] || [ -z "$WPACODE" ] && {
    LOG_MSG "Insufficient information to configure the wireless AP."
    return
  }
  
  # wireless key
  [ $ROHS -eq 1 ] && WIRELESS_KEY="ap:settings/ssid0" || WIRELESS_KEY="ap:settings"  
  
  # enable wireless AP
  $CMD "$WIRELESS_KEY/ssid=$SSID"
  sleep $SLEEP
  $CMD "ap:settings/ap_enabled=1"
  
  # wait for all sorts of things to happen with the wireless AP configuration
  [ $ROHS -eq 1 ] && sleep 40 || sleep 30
  
  # this for 1350A only
  [ $ROHS -eq 1 ] && {
     $CMD "$WIRELESS_KEY/ssid_type=1"
     sleep $SLEEP
   }
 
  ### configure WPA ##
  
  # common to both kinds of router
  $CMD "$WIRELESS_KEY/privacy_type=1"
  sleep $SLEEP
  $CMD "$WIRELESS_KEY/psktype=1"
  sleep $SLEEP
  $CMD "$WIRELESS_KEY/pskvalue=$WPACODE"
  sleep $SLEEP
  $CMD "$WIRELESS_KEY/wep_privacy=1"
  sleep $SLEEP
  $CMD "$WIRELESS_KEY/wep_auth_type=0"
  sleep $SLEEP
  $CMD "$WIRELESS_KEY/cipher=2"
  
  # wait more
  sleep 10

  # restart wireless with the new settings
  ap_restart.sh
  sleep 10
}

# any user scripts
do_USER()
{
   local T=/var/user.sh
   date
   [ -f $T ] && sh $T
   
   local T=/nvram/user.sh
   [ -f $T ] && sh $T
}

#############
### start ###
#############

# Is there a user.conf? Load it. 
# Its contents will override those passed at the command line
# /var
T=/var/user.conf
[ -f $T ] && {
. $T
}

# /nvram
T=/nvram/user.conf
[ -f $T ] && {
. $T
}

# if verbose then echo the commands (same as "#!/bin/sh -x")
[ "$VERBOSE" = "1" ] && set -o xtrace

# wait for lan to start
waitforlan
sleep 3

# check whether there is a current isp connection
c=0
while [ $c -le 9 ]
do
   tmp=$(cm_getvalue "$PPPOA_KEY/username")
   [ -n "$tmp" ] && break
   c=$(succ $c)
   sleep 3
done

# no connection: configure the default "quickstart"
if [ "$tmp" = "user@ispname" ] || [ "$tmp" = "username" ]; then
  LOG_MSG "New ISP Connection"
  
  # configure the isp connection
  [ "$VERBOSE" = "1" ] && LOG_MSG "Configuring the ISP connection."
  do_ISP
    
  # enable ipaccount ?
  [ "$VERBOSE" = "1" ] && LOG_MSG "Configuring IP account."
  do_IPACCOUNT "$ipaccount"

  # set stuff for setdate, and disable msntp ?
  [ "$VERBOSE" = "1" ] && LOG_MSG "Configuring msntp."
  do_MSNTP "$msntp"
  
  # enable ddns ?
  [ "$VERBOSE" = "1" ] && LOG_MSG "Configuring DDNS."
  do_DDNS "$ddns"
 
  # enable and configure wireless ?
  [ "$VERBOSE" = "1" ] && LOG_MSG "Configuring wireless."
  do_WLAN "$wlan"
  
  # any user script?
  [ "$VERBOSE" = "1" ] && LOG_MSG "Configuring user-defined stuff."
  do_USER
   
  # defrag
  [ "$VERBOSE" = "1" ] && LOG_MSG "Defragmenting the environment."
  setenv DEFRAG DEFRAG
  
  # save changes
  [ "$VERBOSE" = "1" ] && LOG_MSG "Saving changes."
  saveall.sh  
  
else
  if [ "$VERBOSE" = "1" ]; then
     LOG_MSG "There is a current ISP connection: username=$tmp"
     [ "$tmp" != "$ISP_USERNAME" ] && LOG_MSG "Warning - user name mismatch: \"$tmp\" vs \"$ISP_USERNAME\" "
  fi
fi

